@@ -58,6 +58,18 @@ class TestLiteLLMModel:
         assert model4.config["model_list"][0]["litellm_params"]["temperature"] == 0.5
         assert model4.config["model_list"][0]["litellm_params"]["max_tokens"] == 100
 
+        # Test logprobs and top_logprobs configuration passing through (OpenAI-specific)
+        name = CommonLLMNames.OPENAI_TEST.value
+        config_with_logprobs = {
+            "logprobs": True,
+            "top_logprobs": 20,
+            "temperature": 0.7,
+        }
+        model5 = LiteLLMModel(name=name, config=config_with_logprobs)
+        assert model5.config["model_list"][0]["litellm_params"]["logprobs"] is True
+        assert model5.config["model_list"][0]["litellm_params"]["top_logprobs"] == 20
+        assert model5.config["model_list"][0]["litellm_params"]["temperature"] == 0.7
+
     @pytest.mark.vcr(match_on=[*VCR_DEFAULT_MATCH_ON, "body"])
     @pytest.mark.parametrize(
         "config",
@@ -73,11 +85,15 @@ class TestLiteLLMModel:
                                 "temperature": 0,
                                 "max_tokens": 56,
                                 "logprobs": True,
+                                "top_logprobs": 5,
                             },
                         }
                     ],
                 },
-                id="OpenAI-model",
+                id="OpenAI-model-1",
+            ),
+            pytest.param(
+                {"name": CommonLLMNames.GPT_5_MINI.value}, id="OpenAI-model-2"
             ),
             pytest.param(
                 {
@@ -119,12 +135,84 @@ class TestLiteLLMModel:
         if llm.config["model_list"][0]["litellm_params"].get("logprobs"):
             assert isinstance(result.logprob, float)
             assert result.logprob <= 0
+            # Test top_logprobs only for OpenAI models (top_logprobs is OpenAI-specific)
+            if llm.config["model_list"][0]["litellm_params"].get("top_logprobs"):
+                assert isinstance(result.top_logprobs, list)
+                assert len(result.top_logprobs) > 0
+                # Each position should have a list of (token, logprob) tuples
+                for position_logprobs in result.top_logprobs:
+                    assert isinstance(position_logprobs, list)
+                    for token, logprob in position_logprobs:
+                        assert isinstance(token, str)
+                        assert isinstance(logprob, float)
+            else:
+                # For non-OpenAI models or when top_logprobs not configured
+                assert result.top_logprobs is None
         else:
             assert result.logprob is None
+            assert result.top_logprobs is None
         assert result.name == result_name
         result = await llm.call_single(messages)
         assert isinstance(result, LLMResult)
 
+    @pytest.mark.asyncio
+    @pytest.mark.parametrize(
+        "config",
+        [
+            pytest.param(
+                {
+                    "name": CommonLLMNames.OPENAI_TEST.value,
+                    "fallbacks": [
+                        {
+                            CommonLLMNames.OPENAI_TEST.value: [
+                                CommonLLMNames.ANTHROPIC_TEST.value
+                            ]
+                        }
+                    ],
+                    "model_list": [
+                        {
+                            "model_name": CommonLLMNames.OPENAI_TEST.value,
+                            "litellm_params": {
+                                "model": CommonLLMNames.OPENAI_TEST.value,
+                                "api_key": "invalid_key_to_force_fallback",  # pragma: allowlist secret
+                            },
+                        },
+                        {
+                            "model_name": CommonLLMNames.ANTHROPIC_TEST.value,
+                            "litellm_params": {
+                                "model": CommonLLMNames.ANTHROPIC_TEST.value,
+                                "max_tokens": 56,
+                            },
+                        },
+                    ],
+                },
+                id="multiple-models",
+            ),
+        ],
+    )
+    @pytest.mark.vcr(match_on=[*VCR_DEFAULT_MATCH_ON, "body"])
+    async def test_call_w_multiple_models(self, config: dict[str, Any]) -> None:
+        llm = LiteLLMModel(config=config)
+        messages = [
+            Message(role="system", content="Respond with single words."),
+            Message(role="user", content="What is the meaning of the universe?"),
+        ]
+        results = await llm.call(messages)
+        assert isinstance(results, list)
+        assert isinstance(results[0], LLMResult)
+        assert results[0].model == CommonLLMNames.ANTHROPIC_TEST.value, (
+            "Expected Anthropic to be used as fallback"
+        )
+
+        llm.name = CommonLLMNames.ANTHROPIC_TEST.value
+        outputs: list[str] = []
+        results = await llm.call(messages, callbacks=[outputs.append])
+        assert isinstance(results, list)
+        assert isinstance(results[0], LLMResult)
+        assert results[0].model == CommonLLMNames.ANTHROPIC_TEST.value, (
+            "Expected Anthropic to be used after changing llm.name"
+        )
+
     # @pytest.mark.vcr(match_on=[*VCR_DEFAULT_MATCH_ON])
     @pytest.mark.asyncio
     async def test_call_w_figure(self) -> None:
@@ -259,6 +347,15 @@ class TestLiteLLMModel:
             assert completion.cost > 0
             assert completion.completion_count > 100, "Expected a long completion"
 
+        with subtests.test(msg="autowraps message"):
+
+            def mock_call(messages, *_, **__):
+                assert isinstance(messages, list)
+                return [None]
+
+            with patch.object(LiteLLMModel, "call", side_effect=mock_call):
+                await llm.call_single("Test message")
+
     @pytest.mark.vcr
     @pytest.mark.parametrize(
         ("config", "bypassed_router"),
@@ -333,6 +430,88 @@ class TestLiteLLMModel:
         assert llm.config == rehydrated_llm.config
         assert llm.router.deployment_names == rehydrated_llm.router.deployment_names
 
+    @pytest.mark.asyncio
+    async def test_acompletion_iter_logprobs_edge_cases(self) -> None:
+        """Test that acompletion_iter handles various logprobs edge cases gracefully."""
+        model = LiteLLMModel(name=CommonLLMNames.OPENAI_TEST.value)
+        messages = [Message(content="Say hello")]
+
+        def _build_mock_completion(
+            model: str = "test-model",
+            logprobs: Any = None,
+            delta_content: str = "",
+            delta_reasoning_content: str = "hmmm",
+            delta_role: str = "assistant",
+            usage: Any = None,
+        ) -> Mock:
+            return Mock(
+                model=model,
+                choices=[
+                    Mock(
+                        logprobs=logprobs,
+                        delta=Mock(
+                            content=delta_content,
+                            reasoning_content=delta_reasoning_content,
+                            role=delta_role,
+                        ),
+                    )
+                ],
+                usage=usage,
+            )
+
+        # Mock the router to return different logprobs scenarios
+        with patch.object(model, "_router") as mock_router:
+            # Mock completion with None logprobs
+            mock_completion_none = _build_mock_completion(delta_content="Hello")
+
+            # Mock completion with logprobs but no content
+            mock_completion_no_content = _build_mock_completion(
+                logprobs=Mock(content=None),
+                delta_content=" world",
+            )
+
+            # Mock completion with empty content list
+            mock_completion_empty = _build_mock_completion(
+                logprobs=Mock(content=[]), delta_content="!"
+            )
+
+            # Mock completion with valid logprobs
+            mock_completion_valid = _build_mock_completion(
+                logprobs=Mock(content=[Mock(logprob=-0.5)])
+            )
+
+            # Mock completion with usage info
+            mock_completion_usage = _build_mock_completion(
+                usage=Mock(prompt_tokens=10, completion_tokens=5)
+            )
+
+            # Create async generator that yields mock completions
+            async def mock_stream():  # noqa: RUF029
+                async def mock_stream_iter():  # noqa: RUF029
+                    yield mock_completion_none
+                    yield mock_completion_no_content
+                    yield mock_completion_empty
+                    yield mock_completion_valid
+                    yield mock_completion_usage
+
+                return mock_stream_iter()
+
+            mock_router.acompletion.return_value = mock_stream()
+
+            # Test that the method doesn't raise exceptions
+            async_iterable = await model.acompletion_iter(messages)
+            results = [result async for result in async_iterable]
+
+            # Verify we got one final result
+            assert len(results) == 1
+            result = results[0]
+            assert isinstance(result, LLMResult)
+            assert result.text == "Hello world!"
+            assert result.model == "test-model"
+            assert result.logprob == -0.5
+            assert result.prompt_count == 10
+            assert result.completion_count == 5
+
 
 class DummyOutputSchema(BaseModel):
     name: str
@@ -580,7 +759,7 @@ class TestMultipleCompletion:
 
 class TestTooling:
     @pytest.mark.asyncio
-    # @pytest.mark.vcr
+    @pytest.mark.vcr(match_on=[*VCR_DEFAULT_MATCH_ON, "body"])
     async def test_tool_selection(self) -> None:
         model = LiteLLMModel(name=CommonLLMNames.OPENAI_TEST.value, config={"n": 1})
 
@@ -598,7 +777,11 @@ class TestTooling:
         messages = [
             Message(
                 role="system",
-                content="You are a helpful assistant who can use tools to do math. Use a tool if needed. If you don't need a tool, just respond with the answer.",
+                content=(
+                    "You are a helpful assistant who can use tools to do math. Use a"
+                    " tool if needed. If you don't need a tool, just respond with the"
+                    " answer."
+                ),
             ),
             Message(role="user", content="What is double of 8?"),
         ]
@@ -667,6 +850,87 @@ class TestTooling:
             assert isinstance(result.messages[0], ToolRequestMessage)
             assert not result.messages[0].tool_calls
 
+    @pytest.mark.asyncio
+    @pytest.mark.vcr
+    async def test_multi_response_validation(self) -> None:
+        model = LiteLLMModel(name="text-completion-openai/babbage-002")
+        with pytest.raises(ValueError, match="2 results"):
+            # Confirming https://github.com/BerriAI/litellm/issues/12298
+            # does not silently pass through LMI
+            await model.call_single(
+                messages=[
+                    Message(role="system", content="Answer in a concise tone."),
+                    Message(content="What is your name?"),
+                ]
+            )
+
+
+class TestReasoning:
+    @pytest.mark.parametrize(
+        "llm_name",
+        [
+            pytest.param(
+                "deepseek/deepseek-reasoner",
+                id="deepseek-reasoner",
+            ),
+            pytest.param(
+                "openrouter/deepseek/deepseek-r1",
+                id="openrouter-deepseek",
+            ),
+        ],
+    )
+    @pytest.mark.vcr(match_on=[*VCR_DEFAULT_MATCH_ON, "body"])
+    @pytest.mark.asyncio
+    async def test_deepseek_model(self, llm_name: str) -> None:
+        llm = LiteLLMModel(name=llm_name)
+        messages = [
+            Message(
+                role="system",
+                content="Think deeply about the following question and answer it.",
+            ),
+            Message(content="What is the meaning of life?"),
+        ]
+        results = await llm.call(messages)
+        for result in results:
+            assert result.reasoning_content
+
+        outputs: list[str] = []
+        results = await llm.call(messages, callbacks=[outputs.append])
+
+        for i, result in enumerate(results):
+            assert result.reasoning_content
+            assert outputs[i] == result.text
+
+    @pytest.mark.vcr(match_on=[*VCR_DEFAULT_MATCH_ON, "body"])
+    @pytest.mark.asyncio
+    async def test_anthropic_model(self) -> None:
+        llm = LiteLLMModel(
+            # Using 3.7 sonnet for its reasoning capabilities
+            name=CommonLLMNames.CLAUDE_37_SONNET.value,
+            config={
+                "model_list": [
+                    {
+                        "model_name": CommonLLMNames.CLAUDE_37_SONNET.value,
+                        "litellm_params": {
+                            "model": CommonLLMNames.CLAUDE_37_SONNET.value,
+                            "reasoning_effort": "low",
+                        },
+                    }
+                ]
+            },
+        )
+        messages = [
+            Message(
+                role="system",
+                content="Think deeply about the following question and answer it.",
+            ),
+            Message(content="What is the meaning of life?"),
+        ]
+        results = await llm.call(messages)
+        for result in results:
+            assert result.reasoning_content is not None, "Should have reasoning content"
+            assert result.text is not None
+
 
 def test_json_schema_validation() -> None:
     # Invalid JSON
@@ -691,55 +955,3 @@ def test_json_schema_validation() -> None:
     with pytest.raises(JSONSchemaValidationError):
         validate_json_completion(mock_completion2, DummyModel)
     validate_json_completion(mock_completion3, DummyModel)
-
-
-@pytest.mark.vcr(match_on=[*VCR_DEFAULT_MATCH_ON, "body"])
-@pytest.mark.asyncio
-async def test_deepseek_model():
-    llm = LiteLLMModel(
-        name="deepseek/deepseek-reasoner",
-        config={
-            "model_list": [
-                {
-                    "model_name": "deepseek/deepseek-reasoner",
-                    "litellm_params": {
-                        "model": "deepseek/deepseek-reasoner",
-                        "api_base": "https://api.deepseek.com/v1",
-                    },
-                }
-            ]
-        },
-    )
-    messages = [
-        Message(
-            role="system",
-            content="Think deeply about the following question and answer it.",
-        ),
-        Message(content="What is the meaning of life?"),
-    ]
-    results = await llm.call(messages)
-    for result in results:
-        assert result.reasoning_content
-
-    outputs: list[str] = []
-    results = await llm.call(messages, callbacks=[outputs.append])
-    for result in results:
-        assert result.reasoning_content
-
-
-@pytest.mark.vcr
-@pytest.mark.asyncio
-async def test_openrouter_reasoning():
-    llm = LiteLLMModel(name="openrouter/deepseek/deepseek-r1", config={"n": 1})
-    messages = [
-        Message(content="What is the meaning of life?"),
-    ]
-    results = await llm.call(messages, include_reasoning=True)
-    assert results[0].reasoning_content
-
-    outputs: list[str] = []
-    with pytest.raises(
-        NotImplementedError,
-        match=r"Reasoning with OpenRouter via `include_reasoning` is not supported in streaming mode.*",
-    ):
-        await llm.call(messages, include_reasoning=True, callbacks=[outputs.append])
