@@ -13,6 +13,7 @@ from .callbacks import (
     LoggingCallback,
     MeanMetricsCallback,
     RolloutDebugDumpCallback,
+    StoreEnvironmentsCallback,
     StoreTrajectoriesCallback,
     TrajectoryFileCallback,
     TrajectoryMetricsCallback,
@@ -46,6 +47,7 @@ __all__ = [
     "OnlineTrainerConfig",
     "RolloutDebugDumpCallback",
     "RolloutManager",
+    "StoreEnvironmentsCallback",
     "StoreTrajectoriesCallback",
     "TEnvCloneFn",
     "TrajectoryFileCallback",
