@@ -196,6 +196,16 @@ class TrajectoryFileCallback(Callback):
         )
 
 
+class StoreEnvironmentsCallback(Callback):
+    """Callback to store the environment underlying each trajectory."""
+
+    def __init__(self):
+        self.traj_id_to_envs: dict[str, Environment] = {}
+
+    async def before_rollout(self, traj_id: str, env: Environment) -> None:
+        self.traj_id_to_envs[traj_id] = env
+
+
 class RolloutDebugDumpCallback(Callback):
     """Dump JSONL files for each agent and environment step to a directory."""
 
@@ -338,7 +348,7 @@ class TrajectoryMetricsCallback(Callback):
     ) -> None:
         for ds in (ds for ds in self._datasets if ds):
             if self._track_tool_usage:
-                cast(ComputeTrajectoryMetricsMixin, ds).tools_to_track = {
+                cast("ComputeTrajectoryMetricsMixin", ds).tools_to_track = {
                     t.info.name for t in tools
                 }
 
@@ -433,7 +443,7 @@ class WandBLoggingCallback(TrajectoryMetricsCallback):
             return
 
         wandb.log({
-            f"eval/{key}_mean": (sum(vals) / len(vals) if vals else None)
+            f"eval/{key}_mean": sum(vals) / len(vals) if vals else None
             for key, vals in self._eval_metrics.items()
         })
 
