@@ -8,11 +8,10 @@ import inspect
 import logging
 from collections.abc import Awaitable, Callable
 from functools import lru_cache
-from typing import Generic, TypeVar, cast, overload
+from typing import TYPE_CHECKING, Generic, TypeVar, cast, overload
 
 import numpy as np
 import tenacity
-import tree
 from aviary.core import Message, Tool, ToolRequestMessage, is_coroutine_callable
 from lmi import (
     EmbeddingModel,
@@ -24,11 +23,13 @@ from lmi import (
 from lmi import LiteLLMModel as LLMModel
 from pydantic import BaseModel
 
-from .gradient_estimators import assign_constant_grads
 from .memory import Memory, MemoryModel, UIndexMemoryModel
-from .op_utils import CallID, get_call_id, get_training_mode
+from .op_utils import CallID, _lazy_import_tree, get_call_id, get_training_mode
 from .ops import GradInType, Op, OpCtx, ResultOrValue, TOutput_co
 
+if TYPE_CHECKING:
+    import tree
+
 logger = logging.getLogger(__name__)
 
 
@@ -76,6 +77,8 @@ class StopGradOp(IdentityOp[TOutput_co]):
         grad_output: tree.Structure,
         call_id: CallID,
     ) -> GradInType:
+        from .gradient_estimators import assign_constant_grads
+
         return assign_constant_grads(input_args, input_kwargs, None)
 
 
@@ -101,7 +104,7 @@ class ConfigOp(Op[TConfig], Generic[TConfig]):
         call_id: CallID,
     ) -> GradInType:
         # Check that the grad_output structure is consistent with our config
-        tree.assert_same_structure(
+        _lazy_import_tree().assert_same_structure(
             grad_output, ctx.get(call_id, "output").value, check_types=False
         )
 
@@ -186,6 +189,8 @@ class FxnOp(Op[TOutput_co]):
         grad_output: tree.Structure,
         call_id: CallID,
     ) -> GradInType:
+        from .gradient_estimators import assign_constant_grads
+
         return assign_constant_grads(input_args, input_kwargs, 0.0)
 
 
@@ -281,7 +286,10 @@ class LLMCallOp(Op[Message]):
             messages=msgs,
             tools=tools,
             tool_choice=tool_choice,
-            **config,
+            # Since config is shared between our LLMModel and our `call` options
+            # we need to ensure we remove keys which work with LLMModel
+            # but not `call`.
+            **{k: v for k, v in config.items() if k != "router_kwargs"},
         )
         if result.messages is None:
             raise ValueError("No messages returned")
@@ -331,7 +339,7 @@ class LLMCallOp(Op[Message]):
             result = await model.call_single(**kwargs)
 
             try:
-                validated = cast(Callable, self.response_validator)(result)
+                validated = cast("Callable", self.response_validator)(result)
                 if inspect.isawaitable(validated):
                     validated = await validated
             except Exception as e:
@@ -375,7 +383,7 @@ class LLMCallOp(Op[Message]):
         # Partition function estimate:
         # Z_T = E_P[ e^(lnP/T - lnP) ]
         log_Z_T = logsumexp([
-            temp_factor * cast(float, result.logprob) for result in results
+            temp_factor * cast("float", result.logprob) for result in results
         ]) - np.log(self.num_samples_partition_estimate)
 
         return (raw_log_p / temperature) - log_Z_T
@@ -394,7 +402,9 @@ class LLMCallOp(Op[Message]):
         # but not necessarily each message or tool.
 
         # tree.map_structure allows us to assign a gradient of 0 to all fields of config
-        grad_config = tree.map_structure(lambda _: 0.0, input_kwargs["config"])
+        grad_config = _lazy_import_tree().map_structure(
+            lambda _: 0.0, input_kwargs["config"]
+        )
         grad_kwargs = {"config": grad_config}
         for arg in ("msgs", "tools", "tool_choice"):
             if arg in input_kwargs:
@@ -469,6 +479,8 @@ class MemoryOp(Op[list[Memory]]):
         call_id: CallID,
     ) -> GradInType:
         """Backward pass for memory retrieval - goes back to item."""
+        from .gradient_estimators import assign_constant_grads
+
         return assign_constant_grads(input_args, input_kwargs, 0.0)
 
 
