@@ -5,7 +5,7 @@ from enum import IntEnum, auto
 from functools import partial
 from pathlib import Path
 from typing import cast
-from unittest.mock import patch
+from unittest.mock import Mock, patch
 
 import networkx as nx
 import pytest
@@ -99,7 +99,7 @@ class TestAgentState:
         agent_state_0_json = agent_state_0.model_dump_json()
         for _ in range(3):  # Give a few steps to finish, the assertion needs >0 steps
             action, agent_state, _ = await agent.get_asv(agent_state, obs)
-            obs, reward, done, truncated = await dummy_env.step(action.value)
+            obs, reward, done, truncated = await dummy_env.step(action.value)  # noqa: RUF059
             if done:
                 break
         assert done, "Environment should have finished"
@@ -129,7 +129,7 @@ class TestSimpleAgent:
         agent = SimpleAgent(llm_model={"name": model_name, "temperature": 0.1})
         agent_state = await agent.init_state(tools=tools)
         action, agent_state, _ = await agent.get_asv(agent_state, obs)
-        obs, reward, done, truncated = await dummy_env.step(action.value)
+        obs, reward, done, truncated = await dummy_env.step(action.value)  # noqa: RUF059
         assert reward > 0, (
             "Reward should be positive, indicating agent called print_story tool"
         )
@@ -192,7 +192,7 @@ class TestSimpleAgent:
 
         graph = to_network(action)
         with (
-            tempfile.NamedTemporaryFile(mode="w", suffix=".png") as f,
+            tempfile.NamedTemporaryFile(mode="w", suffix=".png", encoding="utf-8") as f,
             contextlib.suppress(
                 FileNotFoundError  # Allow tests to run without graphviz on OS
             ),
@@ -245,8 +245,8 @@ class TestSimpleAgent:
         obs, *_ = await dummy_env.step(action.value)
         _, agent_state_2, _ = await agent.get_asv(agent_state_1, obs)
 
-        orig_action = cast(ToolRequestMessage, agent_state_1.messages[1])
-        modified_action = cast(ToolRequestMessage, agent_state_2.messages[1])
+        orig_action = cast("ToolRequestMessage", agent_state_1.messages[1])
+        modified_action = cast("ToolRequestMessage", agent_state_2.messages[1])
 
         # tool calls shouldn't have been modified
         assert orig_action.tool_calls == modified_action.tool_calls
@@ -280,7 +280,7 @@ class TestNoToolsSimpleAgent:
         assert isinstance(tool_req_msg, ToolRequestMessage)
         assert len(tool_req_msg.tool_calls) == 1
         assert tool_req_msg.tool_calls[0].function.name == print_story_tool.info.name
-        obs, reward, done, truncated = await dummy_env.step(action.value)
+        obs, reward, done, truncated = await dummy_env.step(action.value)  # noqa: RUF059
         assert reward > 0, (
             "Reward should be positive, indicating agent called print_story tool"
         )
@@ -318,7 +318,7 @@ class TestMemoryAgent:
 
         new_action, agent_state, _ = await agent.get_asv(agent_state, obs)
         assert "Once there was" in str(new_action)
-        obs, reward, done, truncated = await dummy_env.step(new_action.value)
+        obs, reward, done, truncated = await dummy_env.step(new_action.value)  # noqa: RUF059
         assert reward > 0, (
             "Reward should be positive, indicating agent called print_story tool"
         )
@@ -343,7 +343,7 @@ class TestMemoryAgent:
         agent_state = await agent.init_state(tools=tools)
         action, agent_state, _ = await agent.get_asv(agent_state, obs)
         assert action.call_id is not None
-        obs, reward, done, truncated = await dummy_env.step(action.value)
+        obs, reward, done, truncated = await dummy_env.step(action.value)  # noqa: RUF059
         assert reward > 0, (
             "Reward should be positive, indicating agent called print_story tool"
         )
@@ -400,7 +400,7 @@ class TestReActAgent:
         )
         agent_state = await agent.init_state(tools=tools)
         action, agent_state, _ = await agent.get_asv(agent_state, obs)
-        obs, reward, done, truncated = await dummy_env.step(action.value)
+        obs, reward, done, truncated = await dummy_env.step(action.value)  # noqa: RUF059
         assert reward > 0, (
             "Reward should be positive, indicating agent called print_story tool"
         )
@@ -438,7 +438,7 @@ class TestReActAgent:
                 # we want to force longer trajectories. In single_prompt mode, parallel tool
                 # calling is not possible, and OpenAI requires parallel_tool_calls=None
                 # if no tools are provided.
-                "parallel_tool_calls": (None if single_prompt else False),
+                "parallel_tool_calls": None if single_prompt else False,
             },
         )
         agent_state = await agent.init_state(tools=tools)
@@ -498,7 +498,7 @@ class TestReActAgent:
         agent_state = await agent.init_state(tools=tools)
         action, agent_state, _ = await agent.get_asv(agent_state, obs)
         assert action.call_id is not None
-        obs, reward, done, truncated = await dummy_env.step(action.value)
+        obs, reward, done, truncated = await dummy_env.step(action.value)  # noqa: RUF059
         assert reward > 0, (
             "Reward should be positive, indicating agent called print_story tool"
         )
@@ -539,7 +539,7 @@ class TestReActAgent:
 
         graph = to_network(action, max_label_height=4, max_label_width=50)
         with (
-            tempfile.NamedTemporaryFile(mode="w", suffix=".png") as f,
+            tempfile.NamedTemporaryFile(mode="w", suffix=".png", encoding="utf-8") as f,
             contextlib.suppress(
                 FileNotFoundError  # Allow tests to run without graphviz on OS
             ),
@@ -568,7 +568,7 @@ class TestReActAgent:
                     " defaulted_float)\n\nDESCRIPTION:\n    Check using docstrings as"
                     " partial f-string templates like so:"
                     " {summary_format}.\n\nPARAMETERS:\n    x (integer): Yes, I end"
-                    " with a colon :\n    y (null): I am null. And despite that there"
+                    " with a colon :\n    y (null): I am null.\nAnd despite that there"
                     " is a multiline argument description.\n    union (integer | null):"
                     " I am a union and the current year is {current_year}.\n   "
                     " pydantic_model (unknown): I am a Pydantic model.\n    basic_dict"
@@ -599,8 +599,8 @@ class TestReActAgent:
                     " docstrings as partial f-string templates like so:"
                     ' {summary_format}.","parameters":{"type":"object","properties":{"x":{"description":"Yes,'
                     " I end with a colon"
-                    ' :","title":"X","type":"integer"},"y":{"description":"I am null.'
-                    " And despite that there is a multiline argument"
+                    ' :","title":"X","type":"integer"},"y":{"description":"I am'
+                    " null.\\nAnd despite that there is a multiline argument"
                     ' description.","title":"Y","type":"null"},"union":{"anyOf":[{"type":"integer"},{"type":"null"}],"description":"I'
                     " am a union and the current year is"
                     ' {current_year}.","title":"Union"},"pydantic_model":{"$ref":"#/$defs/StubState","description":"I'
@@ -646,7 +646,7 @@ class TestReActAgent:
                     " {summary_format}.</description><parameters><type>object</type><properties><x><description>Yes,"
                     " I end with a colon"
                     " :</description><title>X</title><type>integer</type></x><y><description>I"
-                    " am null. And despite that there is a multiline argument"
+                    " am null.\nAnd despite that there is a multiline argument"
                     " description.</description><title>Y</title><type>null</type></y><union><anyOf><item><type>integer</type></item><item><type>null</type></item></anyOf><description>I"
                     " am a union and the current year is"
                     " {current_year}.</description><title>Union</title></union><pydantic_model><key"
@@ -696,7 +696,20 @@ class TestReActAgent:
         ]
         user_msg = Message(content="Cast the string '5.6' to a float.")
         with (
-            patch.object(LLMModel, "acompletion") as mock_acompletion,
+            patch.object(
+                LLMModel,
+                "acompletion",
+                side_effect=[
+                    [
+                        Mock(
+                            # Stub values to resemble one LLMResult
+                            prompt_count=1,
+                            completion_count=1,
+                            messages=[Mock(spec=Message)],
+                        )
+                    ]
+                ],
+            ) as mock_acompletion,
             patch.object(ReActModuleSinglePrompt, "parse_message"),
         ):
             agent = ReActAgent(
@@ -772,7 +785,7 @@ class TestHTTPAgentClient:
             assert isinstance(vhat, float)
 
             # This makes an obs with ToolResponseMessage inside
-            obs, reward, done, _ = await dummy_env.step(action.value)
+            obs, reward, done, _ = await dummy_env.step(action.value)  # noqa: RUF059
             assert not done
 
             with patch("httpx.AsyncClient.post", async_client.post), eval_mode():
@@ -800,6 +813,6 @@ async def test_interactive(dummy_env: DummyEnv, mocker):
     )
 
     action, agent_state, _ = await agent.get_asv(agent_state, obs)
-    next_obs, _, done, _ = await dummy_env.step(action.value)
+    next_obs, _, done, _ = await dummy_env.step(action.value)  # noqa: RUF059
 
     assert mock_input.call_count >= 2
